<?php

namespace NoccyLabs\PulseAudio\Sink;

use NoccyLabs\PulseAudio\Helper\Pactl;
use NoccyLabs\PulseAudio\Helper\Pacmd;
use NoccyLabs\PulseAudio\PulseAudio;
use NoccyLabs\PulseAudio\PropertyList;

class NullSink extends Sink
{
    
    protected $destroyed = false;

    protected $persistent = false;

    protected $moduleIndex;

    public function __construct(PulseAudio $pulse, $name=null)
    {
        $name = $name?:uniqid("null");
        
        $this->moduleIndex = Pactl::call('load-module', [ 'module-null-sink', 'sink_name='.$name ]);
        
        $sinks = Pacmd::query('list-sinks');
        foreach ($sinks as $index=>$sink) {
            if ($sink['name'] == $name) {
                parent::__construct($pulse, $index, $sink);
                return;
            }    
        }

        throw new \RuntimeException("Unable to create null sink!");
    }

    public function __destruct()
    {
        if ($this->persistent) {
            return;
        }
        if (!$this->isDestroyed()) {
            $this->destroySink();
        }
    }

    public function destroySink()
    {
        Pactl::call('unload-module', [ $this->moduleIndex ]);
        $this->destroyed = true;
    }

    public function isDestroyed()
    {
        return $this->destroyed;
    }

    /**
     * Flags the null-sink as persistent, meaning that it will not be
     * automatically destroyed from the NullSink destructor. You can
     * still manually destroy the sink within the object scope.
     *
     * The module-null-sink will have to be manually removed afterwards,
     * see the documentation for more information.
     *
     */
    public function makePersistent()
    {
        $this->persistent = true;
    }

}

